import java.awt.*;
import java.awt.image.BufferedImage;


/**
 * Shape class for drawing a rectangular selection.
 */
public class RectSelection extends SelectionShape {
	/** Constructor: */
	public RectSelection(){
		super();
	}
	
	/**
	 * Constructor:
	 * @param x X position of the top-left corner.
	 * @param y Y position fo the top-left corner.
	 * @param wid Width of the selection.
	 * @param het height of the selection.
	 */
	public RectSelection(int x, int y, int wid, int het){
		super();
		pos.add(new XYPair(x, y));
		pos.add(new XYPair(wid, het));
	}
	
	/**
	 * Constructor:
	 * @param c Color of the selection line.
	 */
	public RectSelection(Color c){
		super(c);
	}

	/**
	 * Constructor:
	 * @param x X position of the top-left corner.
	 * @param y Y position fo the top-left corner.
	 * @param wid Width of the selection.
	 * @param het height of the selection.
	 * @param c Color of the selection line.
	 */
	public RectSelection(int x, int y, int wid, int het, Color c){
		super(c);
		pos.add(new XYPair(x, y));
		pos.add(new XYPair(wid, het));
	}
	
	/** Paints this shape.
	 * @param g Graphics object to paint on.
	 */
	public void paint(Graphics g){
		Color c=g.getColor();
		g.setColor(color);

		int x=getX(), y=getY(), w=getWidth(), h=getHeight();
		// draw the horizontal lines of the selection box.
		for(int i=x; i<x+w; i+=4){
			g.fillRect(i, y, 1, 1);
			g.fillRect(i, y+h, 1, 1);
			if(i+1<x+w){
				g.fillRect(i+1, y, 1, 1);
				g.fillRect(i+1, y+h, 1, 1);
			}
		}
		// draw the vertical lines.
		for(int i=y; i<y+h; i+=4){
			g.fillRect(x, i, 1, 1);
			g.fillRect(x+w, i, 1, 1);
			if(i+1<y+h){
				g.fillRect(x, i+1, 1, 1);
				g.fillRect(x+w, i+1, 1, 1);
			}
		}
		g.setColor(c);
	}
	
	/**
	 * Sets the x position of the selection.
	 * @param x X position.
	 */
	public void setX(int x){
		((XYPair)pos.elementAt(0)).x=x;
	}
	
	/**
	 * Sets the y position of the selection.
	 * @param y Y position.
	 */
	public void setY(int y){
		((XYPair)pos.elementAt(0)).y=y;
	}

	/**
	 * Sets the width of the selection.
	 * @param w Width of the selection.
	 */
	public void setWidth(int w){
		((XYPair)pos.elementAt(1)).x=w;
	}

	/**
	 * Sets the height of the selection.
	 * @param h Height of the selection.
	 */
	public void setHeight(int h){
		((XYPair)pos.elementAt(1)).y=h;
	}

	/**
	 * Returns the x position of the selection.
	 * @return X position.
	 */
	public int getX(){
		return ((XYPair)pos.elementAt(0)).x;
	}

	/**
	 * Returns the y position of the selection.
	 * @return Y position.
	 */
	public int getY(){
		return ((XYPair)pos.elementAt(0)).y;
	}

	/**
	 * Returns the width of the selection.
	 * @return Width of the selection.
	 */
	public int getWidth(){
		return ((XYPair)pos.elementAt(1)).x;
	}

	/**
	 * Returns the height of the selection.
	 * @return Height of the selection.
	 */
	public int getHeight(){
		return ((XYPair)pos.elementAt(1)).y;
	}
	
	/**
	 * Tests if a point is withint the shape.
	 * @param x X coordinate.
	 * @param y Y coordinate.
	 * @return true if it's within the shape, false otherwise.
	 */
	public boolean hitTest(int x, int y){
		// the top-left corner.
		XYPair p=(XYPair)pos.elementAt(0);
		// width and height.
		XYPair d=(XYPair)pos.elementAt(1);
		int tlx=getX(), tly=getY(), w=getWidth(), h=getHeight();
		if(tlx<=x && x<tlx+w && tly<=y && y<tly+h){
			return true;			
		}
		return false;
	}
	
	/**
	 * Returns an image that contains the area of 'im' that's in this shape.
	 * @param im Image to get the selection for.
	 * @return Image containing the area of 'im' that's in this shape.
	 */
	public BufferedImage getSelection(BufferedImage im){
		int w=getWidth(), h=getHeight(), x=getX(), y=getY();
		BufferedImage selection=new BufferedImage(w, h, BufferedImage.TYPE_INT_ARGB);
		selection.createGraphics().drawImage(im, 0, 0, w, h,
			x, y, x+w, y+h, null);
		return selection;
	}
}
